/*
* preSavePageFunction stub:
*
* The name of the function can be changed to anything you like.
*
* The function will be passed one 'options' argument that has the following fields:
*   'data' - an array of records representing one page of data. A record can be an object {} or array [] depending on the data source.
*   'files' - file exports only. files[i] contains source file metadata for data[i]. i.e. files[i].fileMeta.fileName.
*   'errors' - an array of errors where each error has the structure {code: '', message: '', source: '', retryDataKey: ''}.
*   'retryData' - a dictionary object containing the retry data for all errors: {retryDataKey: { data: <record>, stage: '', traceKey: ''}}.
*   '_exportId' - the _exportId currently running.
*   '_connectionId' - the _connectionId currently running.
*   '_flowId' - the _flowId currently running.
*   '_integrationId' - the _integrationId currently running.
*   'pageIndex' - 0 based. context is the batch export currently running.
*   'lastExportDateTime' - delta exports only.
*   'currentExportDateTime' - delta exports only.
*   'settings' - all custom settings in scope for the export currently running.
*
* The function needs to return an object that has the following fields:
*   'data' - your modified data.
*   'errors' - your modified errors.
*   'abort' - instruct the batch export currently running to stop generating new pages of data.
*   'newErrorsAndRetryData' - return brand new errors linked to retry data: [{retryData: <record>, errors: [<error>]}].
* Throwing an exception will signal a fatal error and stop the flow.
*/
function preSavePage (options) {
  // sample code that simply passes on what has been exported
  var adaptors = []
  options.data.forEach((eachData) => {
    if (eachData._exportId) adaptors.push({'type': 'exports', 'id': eachData._exportId, 'flowId': eachData._id, 'flowName': eachData.name, 'integrationId': eachData._integrationId})
    if (eachData._importId) adaptors.push({'type': 'imports', 'id': eachData._importId, 'flowId': eachData._id, 'flowName': eachData.name, 'integrationId': eachData._integrationId})
    if (eachData.pageGenerators) {
      eachData.pageGenerators.forEach((eachAdaptor) => {
        if (!eachAdaptor._exportId) return
        adaptors.push({'type': 'exports', 'id': eachAdaptor._exportId, 'flowId': eachData._id, 'flowName': eachData.name, 'integrationId': eachData._integrationId})
      })
    }
    if (eachData.pageProcessors) {
      eachData.pageProcessors.forEach((eachAdaptor) => {
        adaptors.push({'type': eachAdaptor.type + 's', 'id': eachAdaptor._importId || eachAdaptor._exportId, 'flowId': eachData._id, 'flowName': eachData.name, 'integrationId': eachData._integrationId})
      })
    }
  })
  options.data = adaptors
  return {
    data: options.data,
    errors: options.errors,
    abort: false,
    newErrorsAndRetryData: []
  }
}
/*
* postResponseMapFunction stub:
*
* The name of the function can be changed to anything you like.
*
* The function will be passed one 'options' argument that has the following fields:
*    'postResponseMapData' - an array of records representing the page of data after response mapping is completed. A record can be an object {} or array [] depending on the data source.
*    'responseData' - the array of responses for the page of data.  An individual response will have the following fields:
*        'statusCode' - 200 is a success.  422 is a data error.  403 means the connection went offline.
*        'errors' - [{code: '', message: '', source: ''}]
*        'ignored' - true if the record was filtered/skipped, false otherwise.
*        'data' - exports only.  the array of records returned by the export application.
*        'id' - imports only.  the id from the import application response.
*        '_json' - imports only.  the complete response data from the import application.
*        'dataURI' - imports only.  a URI for the data in the import application (populated only for errored records).
*    'oneToMany' - as configured on your export/import resource.
*    'pathToMany' - as configured on your export/import resource.
*    '_exportId' - the _exportId currently running.
*    '_importId' - the _importId currently running.
*    '_connectionId' - the _connectionId currently running.
*    '_flowId' - the _flowId currently running.
*    '_integrationId' - the _integrationId currently running.
*    'settings' - all custom settings in scope for the export/import currently running.
*
* The function needs to return the postResponseMapData array provided by options.postResponseMapData.  The length of postResponseMapData MUST remain unchanged.  Elements within postResponseMapData can be changed however needed.

* Throwing an exception will signal a fatal error and fail the entire page of records..
*/

function postResponseMap (options) {
  let errorData = []
  options.postResponseMapData[0].errorData.forEach((eachErrorPage) => {
    errorData.push(...eachErrorPage.errors)
  })
  options.postResponseMapData[0].errorData = errorData
  return options.postResponseMapData
}
/*
* preMapFunction stub:
*
* The name of the function can be changed to anything you like.
*
* The function will be passed one ‘options’ argument that has the following fields:
*   ‘data’ - an array of records representing the page of data before it has been mapped.  A record can be an object {} or array [] depending on the data source.
*   '_importId' - the _importId currently running.
*   '_connectionId' - the _connectionId currently running.
*   '_flowId' - the _flowId currently running.
*   '_integrationId' - the _integrationId currently running.
*   'settings' - all custom settings in scope for the import currently running.
*
* The function needs to return an array, and the length MUST match the options.data array length.
* Each element in the array represents the actions that should be taken on the record at that index.
* Each element in the array should have the following fields:
*   'data' - the modified/unmodified record that should be passed along for processing.
*   'errors' -  used to report one or more errors for the specific record.  Each error must have the following structure: {code: '', message: '', source: ‘’ }
* Returning an empty object {} for a specific record will indicate that the record should be ignored.
* Returning both 'data' and 'errors' for a specific record will indicate that the record should be processed but errors should also be logged.
* Throwing an exception will fail the entire page of records.
*/
function preMap (options) {
  options.data.forEach((eachData) => {
    eachData.errorData.forEach((eachError) => {
      if (eachError.retryData) eachError.retryData = JSON.stringify(eachError.retryData)
      let retryDataLink = ""
      if (eachError.retryDataKey) {
        retryDataLink = `https://${options.settings.integration.ioDomain}.io/api/flows/${eachData.flowId}/${eachData.id}/${eachError.retryDataKey}/data`
      }
      eachError.retryDataLink = `https://${options.settings.integration.ioDomain}.io/integrations/` + (eachData.integrationId || `none`) + `/flowBuilder/${eachData.flowId}/errors/${eachData.id}/open/details/${eachError.errorId}/editRetry`
      eachError.retryDataLinkAPI = retryDataLink
    })
  })
  return options.data.map((d) => {
    return {
      data: d
    }
  })
}